/***************************************************************************************
*
* nested IRQ TS201.asm
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This file shows a one-to-one conversion of the SHARC code, provided in the
* SHARC to TigerSHARC Conversion Examples section, to TigerSHARC. 
*
***************************************************************************************/

#include "defTS201.h"  	

.GLOBAL  start;
.GLOBAL  irq0_isr;

.SECTION  program;
start:
/* sets up a temporary stack */
j27 = 0x80100;;
/* Set up interrupt vector table */
j0 = irq0_isr;;
IVIRQ0 = j0;;

/************************* Set IRQ0 to edge sensitive ****************************/	
xr0 = INTCTL;;
xr0 = bclr r0 by INTCTL_IRQ0_EDGE_P;;
IMASKH = xr0;;

/**************************** Enable IRQ0 interrupt ******************************/	
xr0 = IMASKH;;
xr0 = bset r0 by INT_IRQ0_P;;
IMASKH = xr0;;

/************************* Set global interrupt enable ***************************/
SQCTLST = SQCTL_GIE;;

/********************************* Endless loop **********************************/
do_nothing:
NOP;;
NOP;;
NOP;;
NOP;;
JUMP do_nothing;;

Start.END:						start.END:

/**************************** Interrupt Service Routine **************************/
irq0_isr:
// Perform any register saves to stack first
nop;nop;nop;nop;;					
// Enable nested interrupts
[j27+=-4] = RETIB;;
					
// Perform required operation here

// Disable nested interrupts and restore return address from stack 
RETIB = [j27+0x4];;
j27 = j27+4;;		// Modify stack pointer

// Restore any other registers that were saved to the stack here

// Return from interrupt
RTI (ABS)(NP);;	
irq0_isr.END:	
