/***************************************************************************************
*
* user SW exception TS201.asm
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This file shows a one-to-one conversion of the SHARC code, provided in the
* SHARC to TigerSHARC Conversion Examples section, to TigerSHARC. 
*
***************************************************************************************/

#include "defTS201.h"  	

.GLOBAL  start;
.GLOBAL  sft_isr;

.SECTION  program;
start:
/* setup temporary stack for example */
j27 = 0x80100;;

/* Set up interrupt vector table */
j0 = sft_isr;;
IVSW = j0;;

/* Enable SW Exceptions */
SQCTLST = SQCTL_SW;;

/***************************** Generate the exception ****************************/
trap 0x0;;
	
/********************************* Endless loop **********************************/
do_nothing:
NOP;;
NOP;;
JUMP do_nothing;;

start.END:
/**************************** Interrupt Service Routine **************************/
sft_isr:
// Save RETI to stack
[j27+=-4] = RETI;;

// Perform any register saves to stack here

// Extract EXCAUSE field from SQSTAT. If shifter result equals zero then
// it was a trap instruction so extract SPVCMD field. Else exit ISR.
xr0 = SQSTAT;;
xr1 = 0x804;;
xr1 = FEXT r0 by r1;;
if SEQ, jump extract_spvcmd;;
				
exit_sw_isr: 
// Restore any registers that were saved to the stack here
				
// Save return address from sw exception to RETI. 
// Then perform RTI and restore RETI from stack
RETI = RETS;;
j27 = j27+4;;	// Modify stack pointer
RTI; RETI = [j27+j31];;
exit_sw_isr.END:  

// Extract the trap ID. If zero perform required action, else exit ISR.
extract_spvcmd:
xr1  = 0x305;;
xr1 = FEXT r0 by r1;;
if NSEQ, jump exit_sw_isr;;

// Perform required operation here

jump exit_sw_isr;;
extract_spvcmd.END:
sft_isr.END:
